@@ -5,15 +5,14 @@ use warnings;
 
 # Added by eilara as hack around caller() core dump
 # NOTE: Now we've switched to Sub::Uplevel can this be removed? --ADAMK
-use Carp::Heavy                  (); 
-use Carp                         ();
-use Sub::Uplevel                 ();
-use Aspect::Hook                 ();
-use Aspect::Advice::After        ();
-use Aspect::Point::AfterThrowing ();
+use Carp::Heavy           (); 
+use Carp                  ();
+use Sub::Uplevel          ();
+use Aspect::Advice        ();
+use Aspect::AdviceContext ();
 
-our $VERSION = '0.97_05';
-our @ISA     = 'Aspect::Advice::After';
+our $VERSION = '0.35';
+our @ISA     = 'Aspect::Advice';
 
 # NOTE: To simplify debugging of the generated code, all injected string
 # fragments will be defined in $UPPERCASE, and all lexical variables to be
@@ -28,9 +27,8 @@ sub _install {
 	# runtime checks instead of the original.
 	# Because $MATCH_RUN is used in boolean conditionals, if there
 	# is nothing to do the compiler will optimise away the code entirely.
-	my $curried   = $pointcut->curry_runtime;
-	my $compiled  = $curried ? $curried->compiled_runtime : undef;
-	my $MATCH_RUN = $compiled ? '$compiled->()' : 1;
+	my $curried   = $pointcut->curry_run;
+	my $MATCH_RUN = $curried ? '$curried->match_run($name, $runtime)' : 1;
 
 	# When an aspect falls out of scope, we don't attempt to remove
 	# the generated hook code, because it might (for reasons potentially
@@ -63,8 +61,6 @@ sub _install {
 		# Generate the new function
 		no warnings 'redefine';
 		eval <<"END_PERL"; die $@ if $@;
-		package Aspect::Hook;
-
 		*$NAME = sub $PROTOTYPE {
 			# Is this a lexically scoped hook that has finished
 			goto &\$original if $MATCH_DISABLED;
@@ -78,27 +74,37 @@ sub _install {
 				] };
 				return \@\$return unless \$\@;
 
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
+				my \$runtime = {
 					return_value => \$return,
 					exception    => \$\@,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterThrowing';
+				};
+				die \$runtime->{exception} unless $MATCH_RUN;
 
-				die \$_->{exception} unless $MATCH_RUN;
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_throwing',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					original     => \$original,
+					\%\$runtime,
+				);
 
 				# Execute the advice code
-				&\$code(\$_);
+				() = &\$code(\$context);
 
 				# Throw the same (or modified) exception
-				my \$exception = \$_->{exception};
+				my \$exception = \$context->exception;
 				die \$exception if \$exception;
 
 				# Get the (potentially) modified return value
-				return \@{\$_->{return_value}};
+				\$return = \$context->return_value;
+				if ( ref \$return eq 'ARRAY' ) {
+					return \@\$return;
+				} else {
+					return ( \$return );
+				}
 			}
 
 			if ( defined \$wantarray ) {
@@ -109,27 +115,32 @@ sub _install {
 				};
 				return \$return unless \$\@;
 
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
+				my \$runtime = {
 					return_value => \$return,
 					exception    => \$\@,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterThrowing';
+				};
+				die \$runtime->{exception} unless $MATCH_RUN;
 
-				die \$_->{exception} unless $MATCH_RUN;
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_throwing',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					original     => \$original,
+					\%\$runtime,
+				);
 
 				# Execute the advice code
-				&\$code(\$_);
+				my \$dummy = &\$code(\$context);
 
 				# Throw the same (or modified) exception
-				my \$exception = \$_->{exception};
+				my \$exception = \$context->exception;
 				die \$exception if \$exception;
 
-				# Return the potentially-modified value
-				return \$_->{return_value};
+				# Return the potentially-modified return value
+				return \$context->return_value;
 
 			} else {
 				eval {
@@ -139,30 +150,34 @@ sub _install {
 				};
 				return unless \$\@;
 
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
+				my \$runtime = {
 					return_value => undef,
 					exception    => \$\@,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterThrowing';
+				};
+				die \$runtime->{exception} unless $MATCH_RUN;
 
-				die \$_->{exception} unless $MATCH_RUN;
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_throwing',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					original     => \$original,
+					\%\$runtime,
+				);
 
 				# Execute the advice code
-				&\$code(\$_);
+				&\$code(\$context);
 
 				# Throw the same (or modified) exception
-				my \$exception = \$_->{exception};
+				my \$exception = \$context->exception;
 				die \$exception if \$exception;
 
 				return;
 			}
 		};
 END_PERL
-		$self->{installed}++;
 	}
 
 	# If this will run lexical we don't need a descoping hook
@@ -175,70 +190,4 @@ END_PERL
 	return sub { $out_of_scope = 1 };
 }
 
-# Check for pointcut usage not supported by the advice type
-sub _validate {
-	my $self     = shift;
-	my $pointcut = $self->pointcut;
-
-	# Pointcuts using "throwing" are irrelevant in before advice
-	if ( $pointcut->match_contains('Aspect::Pointcut::Returning') ) {
-		return 'The pointcut returning is illegal when used by after_throwing advice';
-	}
-
-	$self->SUPER::_validate(@_);
-}
-
 1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-Aspect::Advice::AfterThrowing - Execute code when a function throws an
-exception
-
-=head1 SYNOPSIS
-
-  use Aspect;
-  
-  after_throwing {
-      # Trace all function calls that throw an exception
-      print STDERR "Called my function " . $_->sub_name . "\n";
-  
-      # Suppress stringwise "bar" errors from foo() and return true instead
-      if (
-          $_->short_name eq 'foo'
-          and
-          ref $_->exception
-          and
-          $_->exception =~ /bar/
-      ) {
-          $_->return_value(1);
-      }
-  
-  } call qr/^ MyModule::\w+ $/
-
-=head1 DESCRIPTION
-
-The C<after_throwing> advice type is used to execute code when calls to a
-function result in it (or something within it) throwing an exception.
-
-The C<after_throwing> advice type can be useful for catching and
-suppressing exceptions like a kind of targetted try/catch mechanism,
-to enhance the information recorded in a exception object, or to apply
-additional logging or other side effects to the exception.
-
-=head1 AUTHORS
-
-Adam Kennedy E<lt>adamk@cpan.orgE<gt>
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright 2010 Adam Kennedy.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
-=cut
